%
% beta1d_um_c.sl
%
% Registers S-Lang routines with Sherpa that will call the
% compiled code from user_model.c/beta1d_model_c.c.
%
% This should be loaded into sherpa by saying
%    require ("beta1d_um_c");
%
% The beta1d_um_c.sl file must be placed in one of the
% directories returned by the get_slang_load_path routine
% (or add the appropriate directory to this path using the
% prepend_to_slang_load_path routine)
%
% The beta1d_um_c-module.so librart must be
% placed in one of the directories returned by the
% get_import_module_path routine (or add the appropriate
% directory using the set_import_module_path routine)
%

% Ensure Sherpa is available
%
require ("sherpa");

% load the compiled routines into S-Lang
%
import ("beta1d_um_c");

% This is the routine that Sherpa calls when it evaluates the
% model
%
public define beta1d_um_c()
{
  % Pop stuff off of stack.  Model parameters
  % are popped off in the same order they 
  % are listed when you call register_model.
  % The appropriate x array(s) were pushed on the
  % stack after the model parameters.
  %
  % If given an integrated dataset just evaluate the
  % model in the center of each bin.
  %
  variable r0, beta, xpos, ampl, x;
  variable xlo, xhi;
  switch (_NARGS)
    {
    case 5:
      (r0, beta, xpos, ampl, x) = ();
    }
    {
    case 6:
      (r0, beta, xpos, ampl, xlo, xhi) = ();
      x = 0.5 * (xlo + xhi);
    }
    {
  	usage ("%s (r0, beta, xpos, ampl, x [, xhi]);",
  	       _function_name);
    }

  % call the C code which returns the calculated values
  %
  return _beta1d_um_c (r0, beta, xpos, ampl, x);
}

%
% register the model with Sherpa:
%    model name
%    array of parameter names
%    number of dimensions the model works on
%    default parameter values
%    minimum parameter values
%    maximum parameter values
%    frozen or thawed (0 or 1 respectively)
%
private variable _flag;
if (0==__is_initialized (&_flag))
  {
    message ("Registering beta1d_um_c as a Sherpa model.");

    % The arguments are
    %   1 - name of model (must match routine name)
    %   2 - array of parameter names
    %   3 - number of dimensions
    %   4 - array of default values
    %   5 - array of minimum values
    %   6 - array of maximum values
    %   7 - array of 1 and 0: 1 means thawed, 0 means frozen
    %
    % Only the first three parameters are required
    %
    if (0 != register_model ("beta1d_um_c",
                             ["r0", "beta", "xpos", "ampl"],
                             1, % 1D model
                             [1.0, 1.0e-5, 0.0, 1.0],
                             [1.0e-120, 1.0e-5, 0.0, 0.0],
                             [1.0e120, 10.0, 1.0e120, 1.0e120],
                             [1, 1, 0, 1]))
      error ("Unable to register model beta1d_um_c!");

    _flag = 1;
  }

provide ("beta1d_um_c");

% End
%
