%
% Create a S-Lang user model that represents the beta1d model.
% Since the model is simple we can create a "pure" S-Lang
% version.
%
% This file can be loaded into Sherpa using either
%   () = evalfile ("beta1d_usermodel.sl");
% or
%   require ("beta1d_usermodel");
%

% Unlike the compiled versions, all we need is a routine that
% evaluates the model, and then we call register_model (see
% 'ahelp register_model') to tell Sherpa about the new model.
%

% Evaluate the beta1D model
%    f(x) = A * (1 + (x-x0)^2 / r0^2)^(-3beta + 1/2)
%
% The name of the routine will be the name used in
% 'source' expressions: for example
%    source = beta1d_um[b1]
%
public define beta1d_um()
{
  % As this is a 1D model we will be sent
  %    r0, beta, xpos, ampl
  % (ie the model parameters) followed by either
  %    x
  % or
  %    xlo, xhi
  % (the latter if the integrate status of the model has
  %  been set to true)
  %
  % For this model we do not want to integrate over the bin,
  % and so we chose to evaluate the model in the center of
  % the bin if xlo, xhi are sent in
  %
  variable r0, beta, xpos, ampl, x, xlo, xhi;
  switch (_NARGS)
    {
    case 5:
      % Grab the arguments
      %
      (r0, beta, xpos, ampl, x) = ();
    }
    {
    case 6:
      % Grab the arguments and calculate the
      % center of each bin
      %
      (r0, beta, xpos, ampl, xlo, xhi) = ();
      x = 0.5 * (xlo + xhi);
    }
    {
      % Incorrect number of arguments
      %
      usage ("%s (r0, beta, xpos, ampl, x [, xhi]);",
	     _function_name);
    }

  % Return the calculated values. Since we use double values
  % here the result will be a Double_Type array so we do not
  % need to do any type conversion.
  %
  % Sherpa can send values outside the limits, so we protect
  % against division by zero here.
  %
  variable r02;
  if (r0 <= 1.0e-5)
    r02 = 1.0e-25;
  else
    r02 = r0 * r0;

  return ampl * (1.0 + (x-xpos)*(x-xpos)/r02)^(0.5 - 3*beta);
}

% Register the model with S-Lang. We only call this once,
% even if the file is loaded multiple times using evalfile.
%
private variable _flag;
if (0 == __is_initialized (&_flag))
  {
    message ("Registering beta1d_um as a Sherpa model.");

    % The arguments are
    %   1 - name of model (must match routine name)
    %   2 - array of parameter names
    %   3 - number of dimensions
    %   4 - array of default values
    %   5 - array of minimum values
    %   6 - array of maximum values
    %   7 - array of 1 and 0: 1 means thawed, 0 means frozen
    %
    % Only the first three parameters are required
    %
    if (0 != register_model ("beta1d_um",
			     ["r0", "beta", "xpos", "ampl"],
			     1, % 1D model
			     [1.0, 1.0e-5, 0.0, 1.0],
			     [1.0e-120, 1.0e-5, 0.0, 0.0],
			     [1.0e120, 10.0, 1.0e120, 1.0e120],
			     [1, 1, 0, 1]))
      error ("Unable to register model beta1d_um!");

    _flag = 1;
  }

provide ("beta1d_usermodel");

