%
% gauss2d_um_cc.sl
%
% Registers S-Lang routines with Sherpa that will call the
% compiled code from user_model.cc/gauss2d_model_cc.cc.
%
% This should be loaded into sherpa by saying
%    require ("gauss2d_um_cc");
%
% The gauss2d_um_cc.sl file must be placed in one of the
% directories returned by the get_slang_load_path routine
% (or add the appropriate directory to this path using the
% prepend_to_slang_load_path routine)
%
% The gauss2d_um_cc-module.so librart must be
% placed in one of the directories returned by the
% get_import_module_path routine (or add the appropriate
% directory using the set_import_module_path routine)
%

% Ensure Sherpa is available
%
require ("sherpa");

% load the compiled routines into S-Lang
%
import ("gauss2d_um_cc");

% This is the routine that Sherpa calls when it evaluates the
% model
%
public define gauss2d_um_cc()
{
  % Pop stuff off of stack.  Model parameters
  % are popped off in the same order they 
  % are listed when you call register_model.
  % The appropriate x array(s) were pushed on the
  % stack after the model parameters.
  %
  % If given an integrated dataset just evaluate the
  % model in the center of each bin.
  %
  variable fwhm, xpos, ypos, ellip, theta, ampl, x0, x1;
  variable xlo, xhi;
  switch (_NARGS)
    {
    case 8:
      (fwhm, xpos, ypos, ellip, theta, ampl, x0, x1) = ();
    }
    {
    case 10:
      % grab lo/hi bins off the parameter stack
      %
      (xlo, xhi) = ();
      x1 = 0.5 * (xlo + xhi);
      (xlo, xhi) = ();
      x0 = 0.5 * (xlo + xhi);

      (fwhm, xpos, ypos, ellip, theta, ampl) = ();
    }
    {
  	usage ("%s (fwhm, xpos, ypos, ellip, theta, ampl, x0 [,x0hi], x1 [,x1hi]);",
  	       _function_name);
    }

  % call the C++ code which returns the calculated values
  %
  return _gauss2d_um_cc (fwhm, xpos, ypos, ellip, theta, ampl, x0, x1);
}

%
% register the model with Sherpa:
%    model name
%    array of parameter names
%    number of dimensions the model works on
%    default parameter values
%    minimum parameter values
%    maximum parameter values
%    frozen or thawed (0 or 1 respectively)
%
private variable _flag;
if (0==__is_initialized (&_flag))
  {
    message ("Registering gauss2d_um_cc as a Sherpa model.");

    % The arguments are
    %   1 - name of model (must match routine name)
    %   2 - array of parameter names
    %   3 - number of dimensions
    %   4 - array of default values
    %   5 - array of minimum values
    %   6 - array of maximum values
    %   7 - array of 1 and 0: 1 means thawed, 0 means frozen
    %
    % Only the first three parameters are required
    %
    if (0 != register_model ("gauss2d_um_cc",
                             ["fwhm", "xpos", "ypos", "ellip", "theta", "ampl"],
                             2, % 2D model
			     [10.0, 0.0, 0.0, 0.0, 0.0, 1.0],
                             [1.0e-120, -1.0e-120, -1.0e-120, 0.0, 0.0, -1.0e-120],
                             [1.0e120, 1.0e120, 1.0e120, 0.999, 6.2832, 1e120],
                             [1, 1, 1, 0, 0, 1]))
      error ("Unable to register model gauss2d_um_cc!");

    _flag = 1;
  }

provide ("gauss2d_um_cc");

% End
%
