%
% Create a S-Lang user model that represents the gauss2d model.
% Since the model is simple we can create a "pure" S-Lang
% version.
%
% This file can be loaded into Sherpa using either
%   () = evalfile ("gauss2d_usermodel.sl");
% or
%   require ("gauss2d_usermodel");
%

% Unlike the compiled versions, all we need is a routine that
% evaluates the model, and then we call register_model (see
% 'ahelp register_model') to tell Sherpa about the new model.
%

% Evaluate the gauss2D model. See 'ahelp gauss2d' for the
% details.
%
% The name of the routine will be the name used in
% 'source' expressions: for example
%    source = gauss2d_um[b1]
%

private variable factor = -2.0 * log(4.0);

public define gauss2d_um()
{
  % As this is a 2D model we will be sent
  %    fwhm, xpos, ypos, ellip, theta, ampl
  % (ie the model parameters) followed by either
  %    x0, x1
  % or
  %    x0lo, x0hi, x1lo, x1hi
  % (the latter if the integrate status of the model has
  %  been set to true)
  %
  % For this model we do not want to integrate over the bin,
  % and so we chose to evaluate the model in the center of
  % the bin.
  %
  variable fwhm, xpos, ypos, ellip, theta, ampl, x0, x1;
  variable xlo, xhi;
  switch (_NARGS)
    {
    case 8:
      % Grab the arguments
      %
      (fwhm, xpos, ypos, ellip, theta, ampl, x0, x1) = ();
    }
    {
    case 10:
      % Grab the lo/hi values for the x1 axis
      %
      (xlo, xhi) = ();
      x1 = 0.5 * (xlo + xhi);
  
      % Grab the lo/hi values for the x0 axis
      %
      (xlo, xhi) = ();
      x0 = 0.5 * (xlo + xhi);
  
      % Grab the parameter values
      %
      (fwhm, xpos, ypos, ellip, theta, ampl) = ();
    }
    {
      % Incorrect number of arguments
      %
      usage ("%s (fwhm, xpos, ypos, ellip, theta, ampl, x0 [, x0hi], x1 [, x1hi]);",
	     _function_name);
    }

  % Return the calculated values. Since we use double values
  % here the result will be a Double_Type array so we do not
  % need to do any type conversion.
  %
  
  variable dx = x0 - xpos;
  variable dy = x1 - ypos;

  variable radius;
  if (ellip > 0.0)
    {
      variable eterm = 1.0 - ellip;

      variable xnew = dx*cos(theta) + dy*sin(theta);
      variable ynew = dy*cos(theta) - dx*sin(theta);
      radius = sqrt (xnew*xnew*eterm*eterm + ynew*ynew) / eterm;
    }
  else
    {
      radius = sqrt(dx*dx + dy*dy);
    }

  radius /= fwhm;
  return ampl * exp (factor * radius * radius);
}

% Register the model with S-Lang. We only call this once,
% even if the file is loaded multiple times using evalfile.
%
private variable _flag;
if (0 == __is_initialized (&_flag))
  {
    message ("Registering gauss2d_um as a Sherpa model.");

    % The arguments are
    %   1 - name of model (must match routine name)
    %   2 - array of parameter names
    %   3 - number of dimensions
    %   4 - array of default values
    %   5 - array of minimum values
    %   6 - array of maximum values
    %   7 - array of 1 and 0: 1 means thawed, 0 means frozen
    %
    % Only the first three parameters are required
    %
    if (0 != register_model ("gauss2d_um",
			     ["fwhm", "xpos", "ypos", "ellip", "theta", "ampl"],
			     2, % 2D model
			     [10.0, 0.0, 0.0, 0.0, 0.0, 1.0],
			     [1.0e-120, -1.0e-120, -1.0e-120, 0.0, 0.0, -1.0e-120],
			     [1.0e120, 1.0e120, 1.0e120, 0.999, 6.2832, 1e120],
			     [1, 1, 1, 0, 0, 1]))
      error ("Unable to register model gauss2d_um!");

    _flag = 1;
  }

provide ("gauss2d_usermodel");

